let active = false;
let numLoaded = 0;

const body = document.getElementById('soundboard');
const soundsContainer = document.getElementById('sounds');

/**
 * @typedef sound
 * @prop {string} name
 * @prop {string} icon
 * @prop {string} icon
 * @prop {"stopped"|"playing"|"fadingout"} status
 * @prop {Audio} player
 * @prop {AudioContext} audioContext
 * @prop {GainNode} gain
 *
 */

const sounds = [
  {
    name: "Toveren",
    icon: "./icons/magie.png",
    sound: "./sounds/magie.mp3",
    fade: true,
  },
  {
    name: "Hondje",
    icon: "./icons/hond.png",
    sound: "./sounds/hond.mp3",
    fade: false,
  },
  {
    name: "Onweer 1",
    icon: "./icons/onweer.png",
    sound: "./sounds/onweer1.mp3",
    fade: true,
  },
  {
    name: "Onweer 2",
    icon: "./icons/onweer.png",
    sound: "./sounds/onweer2.mp3",
    fade: true,
  },
  {
    name: "Paddenstoelen muziek",
    icon: "./icons/efteling-paddenstoel.png",
    sound: "./sounds/efteling-paddenstoel.mp3",
    fade: true,
  },
  {
    name: "Indische waterlelies",
    icon: "./icons/efteling-indische-waterlelies.png",
    sound: "./sounds/efteling-indische-waterlelies.mp3",
    fade: true,
  },
]

const loaded = () => {
  numLoaded++;
  console.log('numLoaded', numLoaded, numLoaded === 1);

  if (numLoaded === sounds.length) {
    console.log('runKlaar');
    klaar();
  }
}

const STATUS_STOPPED = "stopped";
const STATUS_PLAYING = "playing";
const STATUS_FADING_OUT = "fadingout";

const load = () => {
  body.classList.remove('clicktoload');
  body.classList.add('loading');

  sounds.forEach(sound => {
    sound.player = new Audio();
    sound.player.addEventListener('canplaythrough', loaded, {once: true});
    sound.player.src = sound.sound;
    sound.player.onended = () => audioEnded(sound);
    sound.status = STATUS_STOPPED;
  });
}
//body.addEventListener('click', load, {once: true});
load();

const klaar = () => {
  sounds.forEach(sound => {
    const button = document.createElement('button');
    const icon = document.createElement('img');
    icon.setAttribute('src', sound.icon);
    button.append(icon);
    const text = document.createElement('span');
    text.innerText = sound.name;
    button.append(text);
    soundsContainer.append(button);

    button.onclick = () => onClick(sound);
    sound.button = button;
  })

  body.classList.remove('loading');
}

const setStatus = (sound, status) => {
  console.log('SET STATUS', sound.name, sound.status + " => " + status);
  sound.status = status;
  sound.button.classList = [];
  sound.button.classList.add("status-" + status);
}

const onClick = sound => {
  switch (sound.status) {
    case STATUS_STOPPED:
      setStatus(sound, STATUS_PLAYING);

      sound.player.volume = 1;
      sound.player.currentTime = 0;
      sound.player.play();

      break;

    case STATUS_PLAYING:
      setStatus(sound, STATUS_FADING_OUT);

      if (!sound.fade) {
        onClick(sound);
        return;
      }

      const fadeOut = setInterval(() => {
        if (sound.status === STATUS_FADING_OUT) {
          const volume = sound.player.volume * 1000;
          sound.player.volume = Math.round(volume - 1) / 1000;
          if (sound.player.volume < 0.0005) {
            sound.player.pause();
            clearInterval(fadeOut);
            sound.status = STATUS_STOPPED;
            setStatus(sound, STATUS_STOPPED)
          }
        }
      }, 1);


      //sound.gain.gain.exponentialRampToValueAtTime(0.01, sound.audioContext.currentTime + 1);
      break;

    case STATUS_FADING_OUT:
      setStatus(sound, STATUS_STOPPED);
      sound.player.pause();
      sound.player.currentTime = 0;
      break;
  }
}

const audioEnded = sound => {
  setStatus(sound, STATUS_STOPPED);
  sound.player.pause();
  sound.player.currentTime = 0;
}
